#include <board.h>
#include <tempsensor.h>
#include <gpio.h>
#include <delay.h>
#include <lcd.h>
#include <spi.h>
#include <i2c.h>
#include <timer.h>
#include <button.h>
#include <stdint.h>
#include "at45db321d.h"
#include "dac.h"
#include "microdelay.h"

#define PIN1 GPIO2_0
#define PIN2 GPIO2_1
#define PIN3 GPIO2_2
#define PIN4 GPIO2_3
#define KEY1_PRESS !(*P2_DATA & (1 << 8))
#define KEY2_PRESS !(*P2_DATA & (1 << 9))
#define KEY3_PRESS !(*P2_DATA & (1 << 10))

uint8_t 	pin[] = {PIN1, PIN2, PIN3, PIN4};
temperature_t 	temp_t;
int8_t 		critical_temp = 40;
timer_t 	timer;
volatile unsigned int *P2_DATA = (unsigned int *)0x50023FFC;

static unsigned int 	   paddress = 0; 
static unsigned short int  baddress = 0;
static const unsigned int  eadd     = 100; 
static const unsigned char srate    = 8;
static const unsigned char bits     = 8;

/* Timer call back function */
void update_temp(void)
{
	temperature_read(LOCAL, &temp_t);	
}

uint8_t audio_init()
{
	int8_t temp[2];

	spi_init(30000, SPI_CPOL_IDLE_LOW, SPI_CPHASE_LEAD_EDGE, SPI_ENDIAN_MSB_FIRST);
	flashmem_init();
	dac_init();
	microdelay_init();

	/* Audio board initialization */
	ReadManufacturerID(temp);
	if (temp[0] != AUDIO_MAN_ID) {
		lcd_puts("Error in audio board...!\n");
		printf("%2X != %2X", AUDIO_MAN_ID, temp[0]);
		return 0;
	}
	return 1;
}

void play(void)
{
	while (true) {
                if (KEY1_PRESS) 
			break;
                if ((paddress >  eadd) && (baddress == 0)) {
			paddress = 0;
                } else {
			dac_write(ByteRead(paddress,baddress));
			baddress++;
			if (baddress == PGSIZE) {
				baddress = 0;
				paddress++;
			}
			microdelay(srate);
		}
	}
}

void set_threshold(void)
{
	int8_t temp = 30;

	printf("Threshold : %d\n", temp);	
	lcd_puts("Key1  [ + ]\n");
	lcd_puts("Key2  [ - ]\n");
	lcd_puts("Key3  [ Save ]");
	while (1) {
		if (KEY1_PRESS) {
			temp++;
			lcd_clear();
			printf("Threshold : %d\n", temp);
			lcd_puts("Key1  [ + ]\n");
			lcd_puts("Key2  [ - ]\n");
			lcd_puts("Key3  [ Save ]");
		} else if (KEY2_PRESS & (temp > 0)) {
			temp--;
			lcd_clear();
			printf("Threshold : %d\n", temp);
			lcd_puts("Key1  [ + ]\n");
			lcd_puts("Key2  [ - ]\n");
			lcd_puts("Key3  [ Save ]");
		} else if (KEY3_PRESS) {
			break;
		}
	}
	critical_temp = temp;
}

uint8_t temp_init(void)
{
	int8_t temp;

	/* Temperature initialization */
	temp = temperature_init();
	if (-1 == temp) {
		lcd_puts("Error in temperature board\n");
		return 0;
	}
	return 1;
}

void relay_init(void)
{
	int i;
	for (i = 0; i <= 3; i++) {
		gpio_enable_pin(pin[i]);
		gpio_direction_output(pin[i], 0);
		gpio_set_pin(pin[i], 0);
	}
}

void relay_on(uint8_t relay)
{
	gpio_set_pin(pin[relay], 1);	
}

void relay_off(uint8_t relay)
{
	gpio_set_pin(pin[relay], 0);	
}

int main()
{
	board_init();
	board_stdout(LCD_STDOUT);
	lcd_init();
	relay_init();	
	delay_init();
	button_init();
	i2c_init(1000);

	if (!audio_init())
		return 1;

	if (!temp_init())
		return 2;

	set_threshold();
	while (1) {
		mdelay(200);
		update_temp();

		if (temp_t.degree >= critical_temp) {
			lcd_clear();
			printf("High Temp %2d.%2d\n", temp_t.degree, temp_t.milli % 100);
			relay_on(3);
			play();
			relay_off(3);
		} else {
			printf("Temp %2d.%2d\n", temp_t.degree, temp_t.milli % 100);

		}
	}
}
